@@ -1,8 +1,3 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
 from collections import OrderedDict, namedtuple
 from functools import wraps
 import inspect
@@ -119,12 +114,12 @@ class Artist(object):
         self._sketch = rcParams['path.sketch']
         self._path_effects = rcParams['path.effects']
         self._sticky_edges = _XYPair([], [])
+        self._in_layout = True
 
     def __getstate__(self):
         d = self.__dict__.copy()
         # remove the unpicklable remove method, this will get re-added on load
         # (by the axes) if the artist lives on an axes.
-        d['_remove_method'] = None
         d['stale_callback'] = None
         return d
 
@@ -154,7 +149,7 @@ class Artist(object):
             _ax_flag = False
             if hasattr(self, 'axes') and self.axes:
                 # remove from the mouse hit list
-                self.axes.mouseover_set.discard(self)
+                self.axes._mouseover_set.discard(self)
                 # mark the axes as stale
                 self.axes.stale = True
                 # decouple the artist from the axes
@@ -257,6 +252,33 @@ class Artist(object):
         """
         return Bbox([[0, 0], [0, 0]])
 
+    def get_tightbbox(self, renderer):
+        """
+        Like `Artist.get_window_extent`, but includes any clipping.
+
+        Parameters
+        ----------
+        renderer : `.RendererBase` instance
+            renderer that will be used to draw the figures (i.e.
+            ``fig.canvas.get_renderer()``)
+
+        Returns
+        -------
+        bbox : `.BboxBase`
+            containing the bounding box (in figure pixel co-ordinates).
+        """
+
+        bbox = self.get_window_extent(renderer)
+        if self.get_clip_on():
+            clip_box = self.get_clip_box()
+            if clip_box is not None:
+                bbox = Bbox.intersection(bbox, clip_box)
+            clip_path = self.get_clip_path()
+            if clip_path is not None and bbox is not None:
+                clip_path = clip_path.get_fully_transformed_path()
+                bbox = Bbox.intersection(bbox, clip_path.get_extents())
+        return bbox
+
     def add_callback(self, func):
         """
         Adds a callback function that will be called whenever one of
@@ -290,7 +312,7 @@ class Artist(object):
         Fire an event when property changed, calling all of the
         registered callbacks.
         """
-        for oid, func in six.iteritems(self._propobservers):
+        for oid, func in self._propobservers.items():
             func(self)
 
     def is_transform_set(self):
@@ -302,10 +324,11 @@ class Artist(object):
 
     def set_transform(self, t):
         """
-        Set the :class:`~matplotlib.transforms.Transform` instance
-        used by this artist.
+        Set the artist transform.
 
-        ACCEPTS: :class:`~matplotlib.transforms.Transform` instance
+        Parameters
+        ----------
+        t : `.Transform`
         """
         self._transform = t
         self._transformSet = True
@@ -324,6 +347,7 @@ class Artist(object):
             self._transform = self._transform._as_mpl_transform(self.axes)
         return self._transform
 
+    @cbook.deprecated("2.2")
     def hitlist(self, event):
         """
         List the children of the artist which contain the mouse event *event*.
@@ -373,7 +397,9 @@ class Artist(object):
         and *props* is a dictionary of properties you want returned
         with the contains test.
 
-        ACCEPTS: a callable function
+        Parameters
+        ----------
+        picker : callable
         """
         self._contains = picker
 
@@ -450,46 +476,46 @@ class Artist(object):
             artist, return *hit=True* and props is a dictionary of
             properties you want added to the PickEvent attributes.
 
-        ACCEPTS: [None|float|boolean|callable]
+        Parameters
+        ----------
+        picker : None or bool or float or callable
         """
         self._picker = picker
 
     def get_picker(self):
-        'Return the picker object used by this artist'
+        """Return the picker object used by this artist."""
         return self._picker
 
+    @cbook.deprecated("2.2", "artist.figure is not None")
     def is_figure_set(self):
-        """
-        Returns True if the artist is assigned to a
-        :class:`~matplotlib.figure.Figure`.
-        """
+        """Returns whether the artist is assigned to a `.Figure`."""
         return self.figure is not None
 
     def get_url(self):
-        """
-        Returns the url
-        """
+        """Returns the url."""
         return self._url
 
     def set_url(self, url):
         """
-        Sets the url for the artist
+        Sets the url for the artist.
 
-        ACCEPTS: a url string
+        Parameters
+        ----------
+        url : str
         """
         self._url = url
 
     def get_gid(self):
-        """
-        Returns the group id
-        """
+        """Returns the group id."""
         return self._gid
 
     def set_gid(self, gid):
         """
-        Sets the (group) id for the artist
+        Sets the (group) id for the artist.
 
-        ACCEPTS: an id string
+        Parameters
+        ----------
+        gid : str
         """
         self._gid = gid
 
@@ -523,6 +549,10 @@ class Artist(object):
             segments, round to the nearest pixel center
 
         Only supported by the Agg and MacOSX backends.
+
+        Parameters
+        ----------
+        snap : bool or None
         """
         self._snap = snap
         self.stale = True
@@ -535,17 +565,17 @@ class Artist(object):
         -------
         sketch_params : tuple or `None`
 
-        A 3-tuple with the following elements:
+            A 3-tuple with the following elements:
 
-          * `scale`: The amplitude of the wiggle perpendicular to the
-            source line.
+              * `scale`: The amplitude of the wiggle perpendicular to the
+                source line.
 
-          * `length`: The length of the wiggle along the line.
+              * `length`: The length of the wiggle along the line.
 
-          * `randomness`: The scale factor by which the length is
-            shrunken or expanded.
+              * `randomness`: The scale factor by which the length is
+                shrunken or expanded.
 
-        May return `None` if no sketch parameters were set.
+            May return `None` if no sketch parameters were set.
         """
         return self._sketch
 
@@ -568,6 +598,8 @@ class Artist(object):
         randomness : float, optional
             The scale factor by which the length is shrunken or
             expanded (default 16.0)
+
+            .. ACCEPTS: (scale: float, length: float, randomness: float)
         """
         if scale is None:
             self._sketch = None
@@ -576,9 +608,11 @@ class Artist(object):
         self.stale = True
 
     def set_path_effects(self, path_effects):
-        """
-        set path_effects, which should be a list of instances of
-        matplotlib.patheffect._Base class or its derivatives.
+        """Set the path effects.
+
+        Parameters
+        ----------
+        path_effects : `.AbstractPathEffect`
         """
         self._path_effects = path_effects
         self.stale = True
@@ -587,18 +621,16 @@ class Artist(object):
         return self._path_effects
 
     def get_figure(self):
-        """
-        Return the :class:`~matplotlib.figure.Figure` instance the
-        artist belongs to.
-        """
+        """Return the `.Figure` instance the artist belongs to."""
         return self.figure
 
     def set_figure(self, fig):
         """
-        Set the :class:`~matplotlib.figure.Figure` instance the artist
-        belongs to.
+        Set the `.Figure` instance the artist belongs to.
 
-        ACCEPTS: a :class:`matplotlib.figure.Figure` instance
+        Parameters
+        ----------
+        fig : `.Figure`
         """
         # if this is a no-op just return
         if self.figure is fig:
@@ -618,9 +650,11 @@ class Artist(object):
 
     def set_clip_box(self, clipbox):
         """
-        Set the artist's clip :class:`~matplotlib.transforms.Bbox`.
+        Set the artist's clip `.Bbox`.
 
-        ACCEPTS: a :class:`matplotlib.transforms.Bbox` instance
+        Parameters
+        ----------
+        clipbox : `.Bbox`
         """
         self.clipbox = clipbox
         self.pchanged()
@@ -641,9 +675,7 @@ class Artist(object):
         this method will set the clipping box to the corresponding rectangle
         and set the clipping path to ``None``.
 
-        ACCEPTS: [ (:class:`~matplotlib.path.Path`,
-        :class:`~matplotlib.transforms.Transform`) |
-        :class:`~matplotlib.patches.Patch` | None ]
+        ACCEPTS: [(`~matplotlib.path.Path`, `.Transform`) | `.Patch` | None]
         """
         from matplotlib.patches import Patch, Rectangle
 
@@ -697,6 +729,17 @@ class Artist(object):
         "Return the artist's animated state"
         return self._animated
 
+    def get_in_layout(self):
+        """
+        Return boolean flag, ``True`` if artist is included in layout
+        calculations.
+
+        E.g. :doc:`/tutorials/intermediate/constrainedlayout_guide`,
+        `.Figure.tight_layout()`, and
+        ``fig.savefig(fname, bbox_inches='tight')``.
+        """
+        return self._in_layout
+
     def get_clip_on(self):
         'Return whether artist uses clipping'
         return self._clipon
@@ -726,7 +769,9 @@ class Artist(object):
         When False artists will be visible out side of the axes which
         can lead to unexpected results.
 
-        ACCEPTS: [True | False]
+        Parameters
+        ----------
+        b : bool
         """
         self._clipon = b
         # This may result in the callbacks being hit twice, but ensures they
@@ -745,16 +790,18 @@ class Artist(object):
             gc.set_clip_path(None)
 
     def get_rasterized(self):
-        "return True if the artist is to be rasterized"
+        """Return whether the artist is to be rasterized."""
         return self._rasterized
 
     def set_rasterized(self, rasterized):
         """
         Force rasterized (bitmap) drawing in vector backend output.
 
-        Defaults to None, which implies the backend's default behavior
+        Defaults to None, which implies the backend's default behavior.
 
-        ACCEPTS: [True | False | None]
+        Parameters
+        ----------
+        rasterized : bool or None
         """
         if rasterized and not hasattr(self.draw, "_supports_rasterization"):
             warnings.warn("Rasterization of '%s' will be ignored" % self)
@@ -762,13 +809,20 @@ class Artist(object):
         self._rasterized = rasterized
 
     def get_agg_filter(self):
-        "return filter function to be used for agg filter"
+        """Return filter function to be used for agg filter."""
         return self._agg_filter
 
     def set_agg_filter(self, filter_func):
-        """
-        set agg_filter function.
+        """Set the agg filter.
 
+        Parameters
+        ----------
+        filter_func : callable
+            A filter function, which takes a (m, n, 3) float array and a dpi
+            value, and returns a (m, n, 3) array.
+
+            .. ACCEPTS: a filter function, which takes a (m, n, 3) float array
+                and a dpi value, and returns a (m, n, 3) array
         """
         self._agg_filter = filter_func
         self.stale = True
@@ -781,10 +835,11 @@ class Artist(object):
 
     def set_alpha(self, alpha):
         """
-        Set the alpha value used for blending - not supported on
-        all backends.
+        Set the alpha value used for blending - not supported on all backends.
 
-        ACCEPTS: float (0.0 transparent through 1.0 opaque)
+        Parameters
+        ----------
+        alpha : float
         """
         self._alpha = alpha
         self.pchanged()
@@ -792,9 +847,11 @@ class Artist(object):
 
     def set_visible(self, b):
         """
-        Set the artist's visiblity.
+        Set the artist's visibility.
 
-        ACCEPTS: [True | False]
+        Parameters
+        ----------
+        b : bool
         """
         self._visible = b
         self.pchanged()
@@ -804,19 +861,33 @@ class Artist(object):
         """
         Set the artist's animation state.
 
-        ACCEPTS: [True | False]
+        Parameters
+        ----------
+        b : bool
         """
         if self._animated != b:
             self._animated = b
             self.pchanged()
 
+    def set_in_layout(self, in_layout):
+        """
+        Set if artist is to be included in layout calculations,
+        E.g. :doc:`/tutorials/intermediate/constrainedlayout_guide`,
+        `.Figure.tight_layout()`, and
+        ``fig.savefig(fname, bbox_inches='tight')``.
+
+        Parameters
+        ----------
+        in_layout : bool
+        """
+        self._in_layout = in_layout
+
     def update(self, props):
         """
-        Update the properties of this :class:`Artist` from the
-        dictionary *prop*.
+        Update this artist's properties from the dictionary *prop*.
         """
         def _update_property(self, k, v):
-            """sorting out how to update property (setter or setattr)
+            """Sorting out how to update property (setter or setattr).
 
             Parameters
             ----------
@@ -824,6 +895,7 @@ class Artist(object):
                 The name of property to update
             v : obj
                 The value to assign to the property
+
             Returns
             -------
             ret : obj or None
@@ -840,13 +912,8 @@ class Artist(object):
                     raise AttributeError('Unknown property %s' % k)
                 return func(v)
 
-        store = self.eventson
-        self.eventson = False
-        try:
-            ret = [_update_property(self, k, v)
-                   for k, v in props.items()]
-        finally:
-            self.eventson = store
+        with cbook._setattr_cm(self, eventson=False):
+            ret = [_update_property(self, k, v) for k, v in props.items()]
 
         if len(ret):
             self.pchanged()
@@ -854,28 +921,27 @@ class Artist(object):
         return ret
 
     def get_label(self):
-        """
-        Get the label used for this artist in the legend.
-        """
+        """Get the label used for this artist in the legend."""
         return self._label
 
     def set_label(self, s):
         """
         Set the label to *s* for auto legend.
 
-        ACCEPTS: string or anything printable with '%s' conversion.
+        Parameters
+        ----------
+        s : object
+            *s* will be converted to a string by calling `str`.
         """
         if s is not None:
-            self._label = '%s' % (s, )
+            self._label = str(s)
         else:
             self._label = None
         self.pchanged()
         self.stale = True
 
     def get_zorder(self):
-        """
-        Return the :class:`Artist`'s zorder.
-        """
+        """Return the artist's zorder."""
         return self.zorder
 
     def set_zorder(self, level):
@@ -883,8 +949,12 @@ class Artist(object):
         Set the zorder for the artist.  Artists with lower zorder
         values are drawn first.
 
-        ACCEPTS: any number
+        Parameters
+        ----------
+        level : float
         """
+        if level is None:
+            level = self.__class__.zorder
         self.zorder = level
         self.pchanged()
         self.stale = True
@@ -892,7 +962,7 @@ class Artist(object):
     @property
     def sticky_edges(self):
         """
-        `x` and `y` sticky edge lists.
+        ``x`` and ``y`` sticky edge lists for autoscaling.
 
         When performing autoscaling, if a data limit coincides with a value in
         the corresponding sticky_edges list, then no margin will be added--the
@@ -900,8 +970,8 @@ class Artist(object):
         where one usually expects no margin on the bottom edge (0) of the
         histogram.
 
-        This attribute cannot be assigned to; however, the `x` and `y` lists
-        can be modified in place as needed.
+        This attribute cannot be assigned to; however, the ``x`` and ``y``
+        lists can be modified in place as needed.
 
         Examples
         --------
@@ -995,8 +1065,9 @@ class Artist(object):
             data[0]
         except (TypeError, IndexError):
             data = [data]
-        return ', '.join('{:0.3g}'.format(item) for item in data if
-                isinstance(item, (np.floating, np.integer, int, float)))
+        data_str = ', '.join('{:0.3g}'.format(item) for item in data if
+                   isinstance(item, (np.floating, np.integer, int, float)))
+        return "[" + data_str + "]"
 
     @property
     def mouseover(self):
@@ -1009,30 +1080,29 @@ class Artist(object):
         ax = self.axes
         if ax:
             if val:
-                ax.mouseover_set.add(self)
+                ax._mouseover_set.add(self)
             else:
-                ax.mouseover_set.discard(self)
+                ax._mouseover_set.discard(self)
 
 
 class ArtistInspector(object):
     """
-    A helper class to inspect an :class:`~matplotlib.artist.Artist`
-    and return information about it's settable properties and their
-    current values.
+    A helper class to inspect an `~matplotlib.artist.Artist` and return
+    information about its settable properties and their current values.
     """
+
     def __init__(self, o):
+        r"""
+        Initialize the artist inspector with an `Artist` or an iterable of
+        `Artist`\s.  If an iterable is used, we assume it is a homogeneous
+        sequence (all `Artists` are of the same type) and it is your
+        responsibility to make sure this is so.
         """
-        Initialize the artist inspector with an
-        :class:`~matplotlib.artist.Artist` or iterable of :class:`Artists`.
-        If an iterable is used, we assume it is a homogeneous sequence (all
-        :class:`Artists` are of the same type) and it is your responsibility
-        to make sure this is so.
-        """
-        if cbook.iterable(o):
-            # Wrapped in list instead of doing try-except around next(iter(o))
-            o = list(o)
-            if len(o):
-                o = o[0]
+        if not isinstance(o, Artist):
+            if cbook.iterable(o):
+                o = list(o)
+                if len(o):
+                    o = o[0]
 
         self.oorig = o
         if not inspect.isclass(o):
@@ -1043,15 +1113,14 @@ class ArtistInspector(object):
 
     def get_aliases(self):
         """
-        Get a dict mapping *fullname* -> *alias* for each *alias* in
-        the :class:`~matplotlib.artist.ArtistInspector`.
+        Get a dict mapping property fullnames to sets of aliases for each alias
+        in the :class:`~matplotlib.artist.ArtistInspector`.
 
         e.g., for lines::
 
-          {'markerfacecolor': 'mfc',
-           'linewidth'      : 'lw',
+          {'markerfacecolor': {'mfc'},
+           'linewidth'      : {'lw'},
           }
-
         """
         names = [name for name in dir(self.o)
                  if name.startswith(('set_', 'get_'))
@@ -1061,13 +1130,13 @@ class ArtistInspector(object):
             func = getattr(self.o, name)
             if not self.is_alias(func):
                 continue
-            docstring = func.__doc__
-            fullname = docstring[10:]
-            aliases.setdefault(fullname[4:], {})[name[4:]] = None
+            propname = re.search("`({}.*)`".format(name[:4]),  # get_.*/set_.*
+                                 func.__doc__).group(1)
+            aliases.setdefault(propname, set()).add(name[4:])
         return aliases
 
     _get_valid_values_regex = re.compile(
-        r"\n\s*ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))"
+        r"\n\s*(?:\.\.\s+)?ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))"
     )
 
     def get_valid_values(self, attr):
@@ -1075,7 +1144,7 @@ class ArtistInspector(object):
         Get the legal arguments for the setter associated with *attr*.
 
         This is done by querying the docstring of the function *set_attr*
-        for a line that begins with ACCEPTS:
+        for a line that begins with "ACCEPTS" or ".. ACCEPTS":
 
         e.g., for a line linestyle, return
         "[ ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'steps'`` | ``'None'``
@@ -1097,6 +1166,14 @@ class ArtistInspector(object):
         match = self._get_valid_values_regex.search(docstring)
         if match is not None:
             return re.sub("\n *", " ", match.group(1))
+
+        # Much faster than list(inspect.signature(func).parameters)[1],
+        # although barely relevant wrt. matplotlib's total import time.
+        param_name = func.__code__.co_varnames[1]
+        match = re.search("(?m)^ *{} : (.+)".format(param_name), docstring)
+        if match:
+            return match.group(1)
+
         return 'unknown'
 
     def _get_setters_and_targets(self):
@@ -1112,10 +1189,7 @@ class ArtistInspector(object):
             func = getattr(self.o, name)
             if not callable(func):
                 continue
-            if six.PY2:
-                nargs = len(inspect.getargspec(func)[0])
-            else:
-                nargs = len(inspect.getfullargspec(func)[0])
+            nargs = len(inspect.getfullargspec(func).args)
             if nargs < 2 or self.is_alias(func):
                 continue
             source_class = self.o.__module__ + "." + self.o.__name__
@@ -1123,9 +1197,23 @@ class ArtistInspector(object):
                 if name in cls.__dict__:
                     source_class = cls.__module__ + "." + cls.__name__
                     break
+            source_class = self._replace_path(source_class)
             setters.append((name[4:], source_class + "." + name))
         return setters
 
+    def _replace_path(self, source_class):
+        """
+        Changes the full path to the public API path that is used
+        in sphinx. This is needed for links to work.
+        """
+
+        replace_dict = {'_base._AxesBase': 'Axes',
+                      '_axes.Axes': 'Axes'}
+
+        for key, value in replace_dict.items():
+            source_class = source_class.replace(key, value)
+        return source_class
+
     def get_setters(self):
         """
         Get the attribute strings with setters for object.  e.g., for a line,
@@ -1179,8 +1267,8 @@ class ArtistInspector(object):
 
     def pprint_setters(self, prop=None, leadingspace=2):
         """
-        If *prop* is *None*, return a list of strings of all settable properies
-        and their valid values.
+        If *prop* is *None*, return a list of strings of all settable
+        properties and their valid values.
 
         If *prop* is not *None*, it is a valid property name and that
         property will be returned as a string of property : valid
@@ -1205,10 +1293,10 @@ class ArtistInspector(object):
             lines.append('%s%s: %s' % (pad, name, accepts))
         return lines
 
-    def pprint_setters_rest(self, prop=None, leadingspace=2):
+    def pprint_setters_rest(self, prop=None, leadingspace=4):
         """
-        If *prop* is *None*, return a list of strings of all settable properies
-        and their valid values.  Format the output for ReST
+        If *prop* is *None*, return a list of strings of all settable
+        properties and their valid values.  Format the output for ReST
 
         If *prop* is not *None*, it is a valid property name and that
         property will be returned as a string of property : valid
@@ -1233,6 +1321,12 @@ class ArtistInspector(object):
 
         col0_len = max(len(n) for n in names)
         col1_len = max(len(a) for a in accepts)
+
+        lines.append('')
+        lines.append(pad + '.. table::')
+        lines.append(pad + '   :class: property-table')
+        pad += '   '
+
         table_formatstr = pad + '=' * col0_len + '   ' + '=' * col1_len
 
         lines.append('')
@@ -1279,7 +1373,7 @@ class ArtistInspector(object):
         """
 
         lines = []
-        for name, val in sorted(six.iteritems(self.properties())):
+        for name, val in sorted(self.properties().items()):
             if getattr(val, 'shape', ()) != () and len(val) > 6:
                 s = str(val[:6]) + '...'
             else:
@@ -1383,7 +1477,7 @@ def setp(obj, *args, **kwargs):
       >>> setp(lines, linewidth=2, color='r')        # python style
     """
 
-    if not cbook.iterable(obj):
+    if isinstance(obj, Artist):
         objs = [obj]
     else:
         objs = list(cbook.flatten(obj))
@@ -1409,24 +1503,37 @@ def setp(obj, *args, **kwargs):
         raise ValueError('The set args must be string, value pairs')
 
     # put args into ordereddict to maintain order
-    funcvals = OrderedDict()
-    for i in range(0, len(args) - 1, 2):
-        funcvals[args[i]] = args[i + 1]
-
-    ret = [o.update(funcvals) for o in objs]
-    ret.extend([o.set(**kwargs) for o in objs])
-    return [x for x in cbook.flatten(ret)]
+    funcvals = OrderedDict((k, v) for k, v in zip(args[::2], args[1::2]))
+    ret = [o.update(funcvals) for o in objs] + [o.set(**kwargs) for o in objs]
+    return list(cbook.flatten(ret))
 
 
-def kwdoc(a):
+def kwdoc(artist):
+    r"""
+    Inspect an `~matplotlib.artist.Artist` class and return
+    information about its settable properties and their current values.
+
+    It use the class `.ArtistInspector`.
+
+    Parameters
+    ----------
+    artist : `~matplotlib.artist.Artist` or an iterable of `Artist`\s
+
+    Returns
+    -------
+    string
+        Returns a string with a list or rst table with the settable properties
+        of the *artist*. The formating depends on the value of
+        :rc:`docstring.hardcopy`. False result in a list that is intended for
+        easy reading as a docstring and True result in a rst table intended
+        for rendering the documentation with sphinx.
+    """
     hardcopy = matplotlib.rcParams['docstring.hardcopy']
     if hardcopy:
-        return '\n'.join(ArtistInspector(a).pprint_setters_rest(
-                         leadingspace=2))
+        return '\n'.join(ArtistInspector(artist).pprint_setters_rest(
+                         leadingspace=4))
     else:
-        return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=2))
+        return '\n'.join(ArtistInspector(artist).pprint_setters(
+                         leadingspace=2))
 
 docstring.interpd.update(Artist=kwdoc(Artist))
-
-_get_axes_msg = """{0} has been deprecated in mpl 1.5, please use the
-axes property.  A removal date has not been set."""
