@@ -10,41 +10,19 @@ Displays Agg images in the browser, with interactivity
 # - `backend_webagg.py` contains a concrete implementation of a basic
 #   application, implemented with tornado.
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-from matplotlib.externals import six
-
-import io
+import datetime
+from io import StringIO
 import json
 import os
-import time
+from pathlib import Path
 import warnings
 
 import numpy as np
+import tornado
 
 from matplotlib.backends import backend_agg
-from matplotlib.figure import Figure
-from matplotlib import backend_bases
-from matplotlib import _png
-
-
-def new_figure_manager(num, *args, **kwargs):
-    """
-    Create a new figure manager instance
-    """
-    FigureClass = kwargs.pop('FigureClass', Figure)
-    thisFig = FigureClass(*args, **kwargs)
-    return new_figure_manager_given_figure(num, thisFig)
-
-
-def new_figure_manager_given_figure(num, figure):
-    """
-    Create a new figure manager instance for the given figure.
-    """
-    canvas = FigureCanvasWebAggCore(figure)
-    manager = FigureManagerWebAgg(canvas, num)
-    return manager
+from matplotlib.backend_bases import _Backend
+from matplotlib import backend_bases, _png
 
 
 # http://www.cambiaresearch.com/articles/15/javascript-char-codes-key-codes
@@ -111,21 +89,21 @@ def _handle_key(key):
     code = int(key[key.index('k') + 1:])
     value = chr(code)
     # letter keys
-    if code >= 65 and code <= 90:
+    if 65 <= code <= 90:
         if 'shift+' in key:
             key = key.replace('shift+', '')
         else:
             value = value.lower()
     # number keys
-    elif code >= 48 and code <= 57:
+    elif 48 <= code <= 57:
         if 'shift+' in key:
             value = ')!@#$%^&*('[int(value)]
             key = key.replace('shift+', '')
     # function keys
-    elif code >= 112 and code <= 123:
+    elif 112 <= code <= 123:
         value = 'f%s' % (code - 111)
     # number pad keys
-    elif code >= 96 and code <= 105:
+    elif 96 <= code <= 105:
         value = '%s' % (code - 96)
     # keys with shift alternatives
     elif code in _SHIFT_LUT and 'shift+' in key:
@@ -143,11 +121,6 @@ class FigureCanvasWebAggCore(backend_agg.FigureCanvasAgg):
     def __init__(self, *args, **kwargs):
         backend_agg.FigureCanvasAgg.__init__(self, *args, **kwargs)
 
-        # A buffer to hold the PNG data for the last frame.  This is
-        # retained so it can be resent to each client without
-        # regenerating it.
-        self._png_buffer = io.BytesIO()
-
         # Set to True when the renderer contains data that is newer
         # than the PNG buffer.
         self._png_is_old = True
@@ -162,23 +135,21 @@ class FigureCanvasWebAggCore(backend_agg.FigureCanvasAgg):
         # to the connected clients.
         self._current_image_mode = 'full'
 
+        # Store the DPI ratio of the browser.  This is the scaling that
+        # occurs automatically for all images on a HiDPI display.
+        self._dpi_ratio = 1
+
     def show(self):
         # show the figure window
         from matplotlib.pyplot import show
         show()
 
     def draw(self):
-        renderer = self.get_renderer(cleared=True)
-
         self._png_is_old = True
-
-        backend_agg.RendererAgg.lock.acquire()
         try:
-            self.figure.draw(renderer)
+            super().draw()
         finally:
-            backend_agg.RendererAgg.lock.release()
-            # Swap the frames
-            self.manager.refresh_all()
+            self.manager.refresh_all()  # Swap the frames.
 
     def draw_idle(self):
         self.send_event("draw")
@@ -206,8 +177,8 @@ class FigureCanvasWebAggCore(backend_agg.FigureCanvasAgg):
             # The buffer is created as type uint32 so that entire
             # pixels can be compared in one numpy call, rather than
             # needing to compare each plane separately.
-            buff = np.frombuffer(renderer.buffer_rgba(), dtype=np.uint32)
-            buff.shape = (renderer.height, renderer.width)
+            buff = (np.frombuffer(renderer.buffer_rgba(), dtype=np.uint32)
+                    .reshape((renderer.height, renderer.width)))
 
             # If any pixels have transparency, we need to force a full
             # draw as we cannot overlay new on top of old.
@@ -218,31 +189,24 @@ class FigureCanvasWebAggCore(backend_agg.FigureCanvasAgg):
                 output = buff
             else:
                 self.set_image_mode('diff')
-                last_buffer = np.frombuffer(self._last_renderer.buffer_rgba(),
-                                            dtype=np.uint32)
-                last_buffer.shape = (renderer.height, renderer.width)
-
+                last_buffer = (np.frombuffer(self._last_renderer.buffer_rgba(),
+                                             dtype=np.uint32)
+                               .reshape((renderer.height, renderer.width)))
                 diff = buff != last_buffer
                 output = np.where(diff, buff, 0)
 
-            # Clear out the PNG data buffer rather than recreating it
-            # each time.  This reduces the number of memory
-            # (de)allocations.
-            self._png_buffer.truncate()
-            self._png_buffer.seek(0)
-
             # TODO: We should write a new version of write_png that
             # handles the differencing inline
-            _png.write_png(
+            buff = _png.write_png(
                 output.view(dtype=np.uint8).reshape(output.shape + (4,)),
-                self._png_buffer)
+                None, compression=6, filter=_png.PNG_FILTER_NONE)
 
             # Swap the renderer frames
             self._renderer, self._last_renderer = (
                 self._last_renderer, renderer)
             self._force_full = False
             self._png_is_old = False
-        return self._png_buffer.getvalue()
+            return buff
 
     def get_renderer(self, cleared=None):
         # Mirrors super.get_renderer, but caches the old one
@@ -271,81 +235,90 @@ class FigureCanvasWebAggCore(backend_agg.FigureCanvasAgg):
         return self._renderer
 
     def handle_event(self, event):
+        e_type = event['type']
+        handler = getattr(self, 'handle_{0}'.format(e_type),
+                          self.handle_unknown_event)
+        return handler(event)
+
+    def handle_unknown_event(self, event):
+        warnings.warn('Unhandled message type {0}. {1}'.format(
+            event['type'], event), stacklevel=2)
+
+    def handle_ack(self, event):
+        # Network latency tends to decrease if traffic is flowing
+        # in both directions.  Therefore, the browser sends back
+        # an "ack" message after each image frame is received.
+        # This could also be used as a simple sanity check in the
+        # future, but for now the performance increase is enough
+        # to justify it, even if the server does nothing with it.
+        pass
+
+    def handle_draw(self, event):
+        self.draw()
+
+    def _handle_mouse(self, event):
+        x = event['x']
+        y = event['y']
+        y = self.get_renderer().height - y
+
+        # Javascript button numbers and matplotlib button numbers are
+        # off by 1
+        button = event['button'] + 1
+
+        # The right mouse button pops up a context menu, which
+        # doesn't work very well, so use the middle mouse button
+        # instead.  It doesn't seem that it's possible to disable
+        # the context menu in recent versions of Chrome.  If this
+        # is resolved, please also adjust the docstring in MouseEvent.
+        if button == 2:
+            button = 3
+
         e_type = event['type']
         guiEvent = event.get('guiEvent', None)
+        if e_type == 'button_press':
+            self.button_press_event(x, y, button, guiEvent=guiEvent)
+        elif e_type == 'button_release':
+            self.button_release_event(x, y, button, guiEvent=guiEvent)
+        elif e_type == 'motion_notify':
+            self.motion_notify_event(x, y, guiEvent=guiEvent)
+        elif e_type == 'figure_enter':
+            self.enter_notify_event(xy=(x, y), guiEvent=guiEvent)
+        elif e_type == 'figure_leave':
+            self.leave_notify_event()
+        elif e_type == 'scroll':
+            self.scroll_event(x, y, event['step'], guiEvent=guiEvent)
+    handle_button_press = handle_button_release = handle_motion_notify = \
+        handle_figure_enter = handle_figure_leave = handle_scroll = \
+        _handle_mouse
 
-        if e_type == 'ack':
-            # Network latency tends to decrease if traffic is flowing
-            # in both directions.  Therefore, the browser sends back
-            # an "ack" message after each image frame is received.
-            # This could also be used as a simple sanity check in the
-            # future, but for now the performance increase is enough
-            # to justify it, even if the server does nothing with it.
-            pass
-        elif e_type == 'draw':
-            self.draw()
-        elif e_type in ('button_press', 'button_release', 'motion_notify',
-                        'figure_enter', 'figure_leave', 'scroll'):
-            x = event['x']
-            y = event['y']
-            y = self.get_renderer().height - y
+    def _handle_key(self, event):
+        key = _handle_key(event['key'])
+        e_type = event['type']
+        guiEvent = event.get('guiEvent', None)
+        if e_type == 'key_press':
+            self.key_press_event(key, guiEvent=guiEvent)
+        elif e_type == 'key_release':
+            self.key_release_event(key, guiEvent=guiEvent)
+    handle_key_press = handle_key_release = _handle_key
 
-            # Javascript button numbers and matplotlib button numbers are
-            # off by 1
-            button = event['button'] + 1
+    def handle_toolbar_button(self, event):
+        # TODO: Be more suspicious of the input
+        getattr(self.toolbar, event['name'])()
 
-            # The right mouse button pops up a context menu, which
-            # doesn't work very well, so use the middle mouse button
-            # instead.  It doesn't seem that it's possible to disable
-            # the context menu in recent versions of Chrome.  If this
-            # is resolved, please also adjust the docstring in MouseEvent.
-            if button == 2:
-                button = 3
-
-            if e_type == 'button_press':
-                self.button_press_event(x, y, button, guiEvent=guiEvent)
-            elif e_type == 'button_release':
-                self.button_release_event(x, y, button, guiEvent=guiEvent)
-            elif e_type == 'motion_notify':
-                self.motion_notify_event(x, y, guiEvent=guiEvent)
-            elif e_type == 'figure_enter':
-                self.enter_notify_event(xy=(x, y), guiEvent=guiEvent)
-            elif e_type == 'figure_leave':
-                self.leave_notify_event()
-            elif e_type == 'scroll':
-                self.scroll_event(x, y, event['step'], guiEvent=guiEvent)
-        elif e_type in ('key_press', 'key_release'):
-            key = _handle_key(event['key'])
-            if e_type == 'key_press':
-                self.key_press_event(key, guiEvent=guiEvent)
-            elif e_type == 'key_release':
-                self.key_release_event(key, guiEvent=guiEvent)
-        elif e_type == 'toolbar_button':
-            # TODO: Be more suspicious of the input
-            getattr(self.toolbar, event['name'])()
-        elif e_type == 'refresh':
-            figure_label = self.figure.get_label()
-            if not figure_label:
-                figure_label = "Figure {0}".format(self.manager.num)
-            self.send_event('figure_label', label=figure_label)
-            self._force_full = True
-            self.draw_idle()
-
-        else:
-            handler = getattr(self, 'handle_{0}'.format(e_type), None)
-            if handler is None:
-                import warnings
-                warnings.warn('Unhandled message type {0}. {1}'.format(
-                                                        e_type, event))
-            else:
-                return handler(event)
+    def handle_refresh(self, event):
+        figure_label = self.figure.get_label()
+        if not figure_label:
+            figure_label = "Figure {0}".format(self.manager.num)
+        self.send_event('figure_label', label=figure_label)
+        self._force_full = True
+        self.draw_idle()
 
     def handle_resize(self, event):
         x, y = event.get('width', 800), event.get('height', 800)
-        x, y = int(x), int(y)
+        x, y = int(x) * self._dpi_ratio, int(y) * self._dpi_ratio
         fig = self.figure
         # An attempt at approximating the figure size in pixels.
-        fig.set_size_inches(x / fig.dpi, y / fig.dpi)
+        fig.set_size_inches(x / fig.dpi, y / fig.dpi, forward=False)
 
         _, _, w, h = self.figure.bbox.bounds
         # Acknowledge the resize, and force the viewer to update the
@@ -359,20 +332,20 @@ class FigureCanvasWebAggCore(backend_agg.FigureCanvasAgg):
         # The client requests notification of what the current image mode is.
         self.send_event('image_mode', mode=self._current_image_mode)
 
+    def handle_set_dpi_ratio(self, event):
+        dpi_ratio = event.get('dpi_ratio', 1)
+        if dpi_ratio != self._dpi_ratio:
+            # We don't want to scale up the figure dpi more than once.
+            if not hasattr(self.figure, '_original_dpi'):
+                self.figure._original_dpi = self.figure.dpi
+            self.figure.dpi = dpi_ratio * self.figure._original_dpi
+            self._dpi_ratio = dpi_ratio
+            self._force_full = True
+            self.draw_idle()
+
     def send_event(self, event_type, **kwargs):
         self.manager._send_event(event_type, **kwargs)
 
-    def start_event_loop(self, timeout):
-        backend_bases.FigureCanvasBase.start_event_loop_default(
-            self, timeout)
-    start_event_loop.__doc__ = \
-        backend_bases.FigureCanvasBase.start_event_loop_default.__doc__
-
-    def stop_event_loop(self):
-        backend_bases.FigureCanvasBase.stop_event_loop_default(self)
-    stop_event_loop.__doc__ = \
-        backend_bases.FigureCanvasBase.stop_event_loop_default.__doc__
-
 
 _JQUERY_ICON_CLASSES = {
     'home': 'ui-icon ui-icon-home',
@@ -409,9 +382,6 @@ class NavigationToolbar2WebAgg(backend_bases.NavigationToolbar2):
             self.canvas.send_event("cursor", cursor=cursor)
         self.cursor = cursor
 
-    def dynamic_update(self):
-        self.canvas.draw_idle()
-
     def draw_rubberband(self, event, x0, y0, x1, y1):
         self.canvas.send_event(
             "rubberband", x0=x0, y0=y0, x1=x1, y1=y1)
@@ -444,7 +414,9 @@ class FigureManagerWebAgg(backend_bases.FigureManagerBase):
         return toolbar
 
     def resize(self, w, h):
-        self._send_event('resize', size=(w, h))
+        self._send_event(
+            'resize',
+            size=(w / self.canvas._dpi_ratio, h / self.canvas._dpi_ratio))
 
     def set_window_title(self, title):
         self._send_event('figure_label', label=title)
@@ -470,21 +442,19 @@ class FigureManagerWebAgg(backend_bases.FigureManagerBase):
     def refresh_all(self):
         if self.web_sockets:
             diff = self.canvas.get_diff_image()
-            for s in self.web_sockets:
-                s.send_binary(diff)
+            if diff is not None:
+                for s in self.web_sockets:
+                    s.send_binary(diff)
 
     @classmethod
     def get_javascript(cls, stream=None):
         if stream is None:
-            output = io.StringIO()
+            output = StringIO()
         else:
             output = stream
 
-        with io.open(os.path.join(
-                os.path.dirname(__file__),
-                "web_backend",
-                "mpl.js"), encoding='utf8') as fd:
-            output.write(fd.read())
+        output.write((Path(__file__).parent / "web_backend/js/mpl.js")
+                     .read_text(encoding="utf-8"))
 
         toolitems = []
         for name, tooltip, image, method in cls.ToolbarCls.toolitems:
@@ -515,7 +485,44 @@ class FigureManagerWebAgg(backend_bases.FigureManagerBase):
         return os.path.join(os.path.dirname(__file__), 'web_backend')
 
     def _send_event(self, event_type, **kwargs):
-        payload = {'type': event_type}
-        payload.update(kwargs)
+        payload = {'type': event_type, **kwargs}
         for s in self.web_sockets:
             s.send_json(payload)
+
+
+class TimerTornado(backend_bases.TimerBase):
+    def _timer_start(self):
+        self._timer_stop()
+        if self._single:
+            ioloop = tornado.ioloop.IOLoop.instance()
+            self._timer = ioloop.add_timeout(
+                datetime.timedelta(milliseconds=self.interval),
+                self._on_timer)
+        else:
+            self._timer = tornado.ioloop.PeriodicCallback(
+                self._on_timer,
+                self.interval)
+            self._timer.start()
+
+    def _timer_stop(self):
+        if self._timer is None:
+            return
+        elif self._single:
+            ioloop = tornado.ioloop.IOLoop.instance()
+            ioloop.remove_timeout(self._timer)
+        else:
+            self._timer.stop()
+
+        self._timer = None
+
+    def _timer_set_interval(self):
+        # Only stop and restart it if the timer has already been started
+        if self._timer is not None:
+            self._timer_stop()
+            self._timer_start()
+
+
+@_Backend.export
+class _BackendWebAggCoreAgg(_Backend):
+    FigureCanvas = FigureCanvasWebAggCore
+    FigureManager = FigureManagerWebAgg
