@@ -1,21 +1,19 @@
 """
 These are classes to support contour plotting and labelling for the Axes class.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-from six.moves import xrange
 
+from numbers import Integral
 import warnings
-import matplotlib as mpl
+
 import numpy as np
 from numpy import ma
+
+import matplotlib as mpl
 import matplotlib._contour as _contour
 import matplotlib.path as mpath
 import matplotlib.ticker as ticker
 import matplotlib.cm as cm
-import matplotlib.colors as colors
+import matplotlib.colors as mcolors
 import matplotlib.collections as mcoll
 import matplotlib.font_manager as font_manager
 import matplotlib.text as text
@@ -52,44 +50,48 @@ class ClabelText(text.Text):
 
 
 class ContourLabeler(object):
-    """Mixin to provide labelling capability to ContourSet"""
+    """Mixin to provide labelling capability to `.ContourSet`."""
 
-    def clabel(self, *args, **kwargs):
+    def clabel(self, *args,
+               fontsize=None, inline=True, inline_spacing=5, fmt='%1.3f',
+               colors=None, use_clabeltext=False, manual=False,
+               rightside_up=True):
         """
         Label a contour plot.
 
         Call signature::
 
-          clabel(cs, **kwargs)
+          clabel(cs, [levels,] **kwargs)
 
         Adds labels to line contours in *cs*, where *cs* is a
         :class:`~matplotlib.contour.ContourSet` object returned by
-        contour.
-
-        ::
-
-          clabel(cs, v, **kwargs)
-
-        only labels contours listed in *v*.
+        ``contour()``.
 
         Parameters
         ----------
+        cs : `.ContourSet`
+            The ContourSet to label.
+
+        levels : array-like, optional
+            A list of level values, that should be labeled. The list must be
+            a subset of ``cs.levels``. If not given, all levels are labeled.
+
         fontsize : string or float, optional
             Size in points or relative size e.g., 'smaller', 'x-large'.
-            See `Text.set_size` for accepted string values.
+            See `.Text.set_size` for accepted string values.
 
-        colors :
-            Color of each label
+        colors : color-spec, optional
+            The label colors:
 
-            - if *None*, the color of each label matches the color of
-              the corresponding contour
+            - If *None*, the color of each label matches the color of
+              the corresponding contour.
 
-            - if one string color, e.g., *colors* = 'r' or *colors* =
-              'red', all labels will be plotted in this color
+            - If one string color, e.g., *colors* = 'r' or *colors* =
+              'red', all labels will be plotted in this color.
 
-            - if a tuple of matplotlib color args (string, float, rgb, etc),
+            - If a tuple of matplotlib color args (string, float, rgb, etc),
               different labels will be plotted in different colors in the order
-              specified
+              specified.
 
         inline : bool, optional
             If ``True`` the underlying contour is removed where the label is
@@ -131,10 +133,15 @@ class ContourLabeler(object):
             or minus 90 degrees from level. Default is ``True``.
 
         use_clabeltext : bool, optional
-            If ``True``, `ClabelText` class (instead of `Text`) is used to
+            If ``True``, `.ClabelText` class (instead of `.Text`) is used to
             create labels. `ClabelText` recalculates rotation angles
             of texts during the drawing time, therefore this can be used if
             aspect of the axes changes. Default is ``False``.
+
+        Returns
+        -------
+        labels
+            A list of `.Text` instances for the labels.
         """
 
         """
@@ -150,21 +157,15 @@ class ContourLabeler(object):
         `BlockingContourLabeler` (case of manual label placement).
         """
 
-        fontsize = kwargs.get('fontsize', None)
-        inline = kwargs.get('inline', 1)
-        inline_spacing = kwargs.get('inline_spacing', 5)
-        self.labelFmt = kwargs.get('fmt', '%1.3f')
-        _colors = kwargs.get('colors', None)
+        self.labelFmt = fmt
+        self._use_clabeltext = use_clabeltext
+        # Detect if manual selection is desired and remove from argument list.
+        self.labelManual = manual
+        self.rightside_up = rightside_up
 
-        self._use_clabeltext = kwargs.get('use_clabeltext', False)
-
-        # Detect if manual selection is desired and remove from argument list
-        self.labelManual = kwargs.get('manual', False)
-
-        self.rightside_up = kwargs.get('rightside_up', True)
         if len(args) == 0:
             levels = self.levels
-            indices = list(xrange(len(self.cvalues)))
+            indices = list(range(len(self.cvalues)))
         elif len(args) == 1:
             levlabs = list(args[0])
             indices, levels = [], []
@@ -185,14 +186,14 @@ class ContourLabeler(object):
         font_size_pts = self.labelFontProps.get_size_in_points()
         self.labelFontSizeList = [font_size_pts] * len(levels)
 
-        if _colors is None:
+        if colors is None:
             self.labelMappable = self
             self.labelCValueList = np.take(self.cvalues, self.labelIndiceList)
         else:
-            cmap = colors.ListedColormap(_colors, N=len(self.labelLevelList))
-            self.labelCValueList = list(xrange(len(self.labelLevelList)))
+            cmap = mcolors.ListedColormap(colors, N=len(self.labelLevelList))
+            self.labelCValueList = list(range(len(self.labelLevelList)))
             self.labelMappable = cm.ScalarMappable(cmap=cmap,
-                                                   norm=colors.NoNorm())
+                                                   norm=mcolors.NoNorm())
 
         self.labelXYs = []
 
@@ -212,16 +213,16 @@ class ContourLabeler(object):
         else:
             self.labels(inline, inline_spacing)
 
-        # Hold on to some old attribute names.  These are deprecated and will
-        # be removed in the near future (sometime after 2008-08-01), but
-        # keeping for now for backwards compatibility
-        self.cl = self.labelTexts
-        self.cl_xy = self.labelXYs
-        self.cl_cvalues = self.labelCValues
-
         self.labelTextsList = cbook.silent_list('text.Text', self.labelTexts)
         return self.labelTextsList
 
+    cl = property(cbook.deprecated("3.0", alternative="labelTexts")(
+        lambda self: self.labelTexts))
+    cl_xy = property(cbook.deprecated("3.0", alternative="labelXYs")(
+        lambda self: self.labelXYs))
+    cl_cvalues = property(cbook.deprecated("3.0", alternative="labelCValues")(
+        lambda self: self.labelCValues))
+
     def print_label(self, linecontour, labelwidth):
         "Return *False* if contours are too short for a label."
         return (len(linecontour) > 10 * labelwidth
@@ -263,7 +264,7 @@ class ContourLabeler(object):
         """
         Return the width of the label in points.
         """
-        if not isinstance(lev, six.string_types):
+        if not isinstance(lev, str):
             lev = self.get_text(lev, fmt)
 
         lev, ismath = text.Text.is_math_text(lev)
@@ -312,15 +313,15 @@ class ContourLabeler(object):
         return lw
 
     def set_label_props(self, label, text, color):
-        "set the label properties - color, fontsize, text"
+        """Set the label properties - color, fontsize, text."""
         label.set_text(text)
         label.set_color(color)
         label.set_fontproperties(self.labelFontProps)
         label.set_clip_box(self.ax.bbox)
 
     def get_text(self, lev, fmt):
-        "get the text of the label"
-        if isinstance(lev, six.string_types):
+        """Get the text of the label."""
+        if isinstance(lev, str):
             return lev
         else:
             if isinstance(fmt, dict):
@@ -520,15 +521,18 @@ class ContourLabeler(object):
         specified transform will be used to translate (x, y) into
         display coordinates.
 
-        *inline*:
-          controls whether the underlying contour is removed or
-          not. Default is *True*.
+        Parameters
+        ----------
+        x, y : float
+            The approximate location of the label.
 
-        *inline_spacing*:
-          space in pixels to leave on each side of label when
-          placing inline.  Defaults to 5.  This spacing will be
-          exact for labels at locations where the contour is
-          straight, less so for labels on curved contours.
+        inline : bool, optional, default: True
+            If *True* remove the segment of the contour beneath the label.
+
+        inline_spacing : int, optional, default: 5
+            Space in pixels to leave on each side of label when placing
+            inline. This spacing will be exact for labels at locations where
+            the contour is straight, less so for labels on curved contours.
         """
 
         if transform is None:
@@ -665,7 +669,7 @@ class ContourLabeler(object):
 
 
 def _find_closest_point_on_leg(p1, p2, p0):
-    """find closest point to p0 on line segment connecting p1 and p2"""
+    """Find the closest point to p0 on line segment connecting p1 and p2."""
 
     # handle degenerate case
     if np.all(p2 == p1):
@@ -691,7 +695,7 @@ def _find_closest_point_on_leg(p1, p2, p0):
 
 def _is_closed_polygon(X):
     """
-    Tests whether first and last object in a sequence are the same.  These are
+    Return whether first and last object in a sequence are the same. These are
     presumably coordinates on a polygonal curve, in which case this function
     tests if that curve is closed.
     """
@@ -735,7 +739,42 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
     """
     Store a set of contour lines or filled regions.
 
-    User-callable method: clabel
+    User-callable method: `~.axes.Axes.clabel`
+
+    Parameters
+    ----------
+    ax : `~.axes.Axes`
+
+    levels : [level0, level1, ..., leveln]
+        A list of floating point numbers indicating the contour
+        levels.
+
+    allsegs : [level0segs, level1segs, ...]
+        List of all the polygon segments for all the *levels*.
+        For contour lines ``len(allsegs) == len(levels)``, and for
+        filled contour regions ``len(allsegs) = len(levels)-1``. The lists
+        should look like::
+
+            level0segs = [polygon0, polygon1, ...]
+            polygon0 = array_like [[x0,y0], [x1,y1], ...]
+
+    allkinds : ``None`` or [level0kinds, level1kinds, ...]
+        Optional list of all the polygon vertex kinds (code types), as
+        described and used in Path. This is used to allow multiply-
+        connected paths such as holes within filled polygons.
+        If not ``None``, ``len(allkinds) == len(allsegs)``. The lists
+        should look like::
+
+            level0kinds = [polygon0kinds, ...]
+            polygon0kinds = [vertexcode0, vertexcode1, ...]
+
+        If *allkinds* is not ``None``, usually all polygons for a
+        particular contour level are grouped together so that
+        ``level0segs = [polygon0]`` and ``level0kinds = [polygon0kinds]``.
+
+    kwargs :
+        Keyword arguments are as described in the docstring of
+        `~.axes.Axes.contour`.
 
     Attributes
     ----------
@@ -753,20 +792,30 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         levels for filled contours.  See :meth:`_process_colors`.
     """
 
-    def __init__(self, ax, *args, **kwargs):
+    def __init__(self, ax, *args,
+                 levels=None, filled=False, linewidths=None, linestyles=None,
+                 alpha=None, origin=None, extent=None,
+                 cmap=None, colors=None, norm=None, vmin=None, vmax=None,
+                 extend='neither', antialiased=None,
+                 **kwargs):
         """
         Draw contour lines or filled regions, depending on
         whether keyword arg *filled* is ``False`` (default) or ``True``.
 
-        The first three arguments must be:
+        Call signature::
 
-          *ax*: axes object.
+            ContourSet(ax, levels, allsegs, [allkinds], **kwargs)
 
-          *levels*: [level0, level1, ..., leveln]
+        Parameters
+        ----------
+        ax :
+            The `~.axes.Axes` object to draw on.
+
+        levels : [level0, level1, ..., leveln]
             A list of floating point numbers indicating the contour
             levels.
 
-          *allsegs*: [level0segs, level1segs, ...]
+        allsegs : [level0segs, level1segs, ...]
             List of all the polygon segments for all the *levels*.
             For contour lines ``len(allsegs) == len(levels)``, and for
             filled contour regions ``len(allsegs) = len(levels)-1``. The lists
@@ -775,7 +824,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
                 level0segs = [polygon0, polygon1, ...]
                 polygon0 = array_like [[x0,y0], [x1,y1], ...]
 
-          *allkinds*: *None* or [level0kinds, level1kinds, ...]
+        allkinds : [level0kinds, level1kinds, ...], optional
             Optional list of all the polygon vertex kinds (code types), as
             described and used in Path. This is used to allow multiply-
             connected paths such as holes within filled polygons.
@@ -789,27 +838,22 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
             particular contour level are grouped together so that
             ``level0segs = [polygon0]`` and ``level0kinds = [polygon0kinds]``.
 
-        Keyword arguments are as described in the docstring of
-        `~.Axes.contour`.
+        **kwargs
+            Keyword arguments are as described in the docstring of
+            `~axes.Axes.contour`.
         """
         self.ax = ax
-        self.levels = kwargs.pop('levels', None)
-        self.filled = kwargs.pop('filled', False)
-        self.linewidths = kwargs.pop('linewidths', None)
-        self.linestyles = kwargs.pop('linestyles', None)
-
+        self.levels = levels
+        self.filled = filled
+        self.linewidths = linewidths
+        self.linestyles = linestyles
         self.hatches = kwargs.pop('hatches', [None])
-
-        self.alpha = kwargs.pop('alpha', None)
-        self.origin = kwargs.pop('origin', None)
-        self.extent = kwargs.pop('extent', None)
-        cmap = kwargs.pop('cmap', None)
-        self.colors = kwargs.pop('colors', None)
-        norm = kwargs.pop('norm', None)
-        vmin = kwargs.pop('vmin', None)
-        vmax = kwargs.pop('vmax', None)
-        self.extend = kwargs.pop('extend', 'neither')
-        self.antialiased = kwargs.pop('antialiased', None)
+        self.alpha = alpha
+        self.origin = origin
+        self.extent = extent
+        self.colors = colors
+        self.extend = extend
+        self.antialiased = antialiased
         if self.antialiased is None and self.filled:
             self.antialiased = False  # eliminate artifacts; we are not
                                       # stroking the boundaries.
@@ -819,14 +863,11 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         self.nchunk = kwargs.pop('nchunk', 0)
         self.locator = kwargs.pop('locator', None)
-        if (isinstance(norm, colors.LogNorm)
+        if (isinstance(norm, mcolors.LogNorm)
                 or isinstance(self.locator, ticker.LogLocator)):
             self.logscale = True
             if norm is None:
-                norm = colors.LogNorm()
-            if self.extend is not 'neither':
-                raise ValueError('extend kwarg does not work yet with log '
-                                 ' scale')
+                norm = mcolors.LogNorm()
         else:
             self.logscale = False
 
@@ -862,13 +903,12 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
             # extend_max case we don't need to worry about passing more colors
             # than ncolors as ListedColormap will clip.
             total_levels = ncolors + int(extend_min) + int(extend_max)
-            if (len(self.colors) == total_levels and
-                    any([extend_min, extend_max])):
+            if len(self.colors) == total_levels and (extend_min or extend_max):
                 use_set_under_over = True
                 if extend_min:
                     i0 = 1
 
-            cmap = colors.ListedColormap(self.colors[i0:None], N=ncolors)
+            cmap = mcolors.ListedColormap(self.colors[i0:None], N=ncolors)
 
             if use_set_under_over:
                 if extend_min:
@@ -981,15 +1021,28 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
     def legend_elements(self, variable_name='x', str_format=str):
         """
-        Return a list of artist and labels suitable for passing through
+        Return a list of artists and labels suitable for passing through
         to :func:`plt.legend` which represent this ContourSet.
 
-        Args:
+        The labels have the form "0 < x <= 1" stating the data ranges which
+        the artists represent.
 
-            *variable_name*: the string used inside the inequality used
-              on the labels
+        Parameters
+        ----------
+        variable_name : str
+            The string used inside the inequality used on the labels.
+
+        str_format : function: float -> str
+            Function used to format the numbers in the labels.
+
+        Returns
+        -------
+        artists : List[`.Artist`]
+            A list of the artists.
+
+        labels : List[str]
+            A list of the labels.
 
-            *str_format*: function used to format the numbers in the labels
         """
         artists = []
         labels = []
@@ -1121,6 +1174,9 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         """
         Select contour levels to span the data.
 
+        The target number of levels, *N*, is used only when the
+        scale is not log and default locator is used.
+
         We need two more levels for filled contours than for
         line contours, because for the latter we need to specify
         the lower and upper boundary of each range. For example,
@@ -1128,6 +1184,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         one contour line, but two filled regions, and therefore
         three levels to provide boundaries for both regions.
         """
+        self._auto = True
         if self.locator is None:
             if self.logscale:
                 self.locator = ticker.LogLocator()
@@ -1135,8 +1192,27 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
                 self.locator = ticker.MaxNLocator(N + 1, min_n_ticks=1)
 
         lev = self.locator.tick_values(self.zmin, self.zmax)
-        self._auto = True
-        return lev
+
+        try:
+            if self.locator._symmetric:
+                return lev
+        except AttributeError:
+            pass
+
+        # Trim excess levels the locator may have supplied.
+        under = np.nonzero(lev < self.zmin)[0]
+        i0 = under[-1] if len(under) else 0
+        over = np.nonzero(lev > self.zmax)[0]
+        i1 = over[0] + 1 if len(over) else len(lev)
+        if self.extend in ('min', 'both'):
+            i0 += 1
+        if self.extend in ('max', 'both'):
+            i1 -= 1
+
+        if i1 - i0 < 3:
+            i0, i1 = 0, len(lev)
+
+        return lev[i0:i1]
 
     def _contour_level_args(self, z, args):
         """
@@ -1149,26 +1225,20 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         self._auto = False
         if self.levels is None:
             if len(args) == 0:
-                lev = self._autolev(7)
+                levels_arg = 7  # Default, hard-wired.
             else:
-                level_arg = args[0]
-                try:
-                    if type(level_arg) == int:
-                        lev = self._autolev(level_arg)
-                    else:
-                        lev = np.asarray(level_arg).astype(np.float64)
-                except:
-                    raise TypeError(
-                        "Last {0} arg must give levels; see help({0})"
-                        .format(fn))
-            self.levels = lev
+                levels_arg = args[0]
         else:
-            self.levels = np.asarray(self.levels).astype(np.float64)
+            levels_arg = self.levels
+        if isinstance(levels_arg, Integral):
+            self.levels = self._autolev(levels_arg)
+        else:
+            self.levels = np.asarray(levels_arg).astype(np.float64)
 
         if not self.filled:
             inside = (self.levels > self.zmin) & (self.levels < self.zmax)
-            self.levels = self.levels[inside]
-            if len(self.levels) == 0:
+            levels_in = self.levels[inside]
+            if len(levels_in) == 0:
                 self.levels = [self.zmin]
                 warnings.warn("No contour levels were found"
                               " within the data range.")
@@ -1193,24 +1263,28 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         # (Colorbar needs this even for line contours.)
         self._levels = list(self.levels)
 
+        if self.logscale:
+            lower, upper = 1e-250, 1e250
+        else:
+            lower, upper = -1e250, 1e250
+
         if self.extend in ('both', 'min'):
-            self._levels.insert(0, min(self.levels[0], self.zmin) - 1)
+            self._levels.insert(0, lower)
         if self.extend in ('both', 'max'):
-            self._levels.append(max(self.levels[-1], self.zmax) + 1)
+            self._levels.append(upper)
         self._levels = np.asarray(self._levels)
 
         if not self.filled:
             self.layers = self.levels
             return
 
-        # layer values are mid-way between levels
-        self.layers = 0.5 * (self._levels[:-1] + self._levels[1:])
-        # ...except that extended layers must be outside the
-        # normed range:
-        if self.extend in ('both', 'min'):
-            self.layers[0] = -1e150
-        if self.extend in ('both', 'max'):
-            self.layers[-1] = 1e150
+        # Layer values are mid-way between levels in screen space.
+        if self.logscale:
+            # Avoid overflow by taking sqrt before multiplying.
+            self.layers = (np.sqrt(self._levels[:-1])
+                           * np.sqrt(self._levels[1:]))
+        else:
+            self.layers = 0.5 * (self._levels[:-1] + self._levels[1:])
 
     def _process_colors(self):
         """
@@ -1247,7 +1321,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
                 if self.extend in ('both', 'max'):
                     i1 += 1
             self.cvalues = list(range(i0, i1))
-            self.set_norm(colors.NoNorm())
+            self.set_norm(mcolors.NoNorm())
         else:
             self.cvalues = self.layers
         self.set_array(self.levels)
@@ -1287,7 +1361,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
                     if lev < eps:
                         tlinestyles[i] = neg_ls
         else:
-            if isinstance(linestyles, six.string_types):
+            if isinstance(linestyles, str):
                 tlinestyles = [linestyles] * Nlev
             elif cbook.iterable(linestyles):
                 tlinestyles = list(linestyles)
@@ -1305,7 +1379,10 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         return self.alpha
 
     def set_alpha(self, alpha):
-        """sets alpha for all ContourSet artists"""
+        """
+        Set the alpha blending value for all ContourSet artists.
+        *alpha* must be between 0 (transparent) and 1 (opaque).
+        """
         self.alpha = alpha
         self.changed()
 
@@ -1340,7 +1417,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         # Nonetheless, improvements could probably be made.
 
         if indices is None:
-            indices = list(xrange(len(self.levels)))
+            indices = list(range(len(self.levels)))
 
         dmin = np.inf
         conmin = None
@@ -1377,7 +1454,7 @@ class QuadContourSet(ContourSet):
     """
     Create and store a set of contour lines or filled regions.
 
-    User-callable method: :meth:`clabel`
+    User-callable method: `~axes.Axes.clabel`
 
     Attributes
     ----------
@@ -1584,70 +1661,48 @@ class QuadContourSet(ContourSet):
     _contour_doc = """
         Plot contours.
 
+        Call signature::
+
+            contour([X, Y,] Z, [levels], **kwargs)
+
         :func:`~matplotlib.pyplot.contour` and
         :func:`~matplotlib.pyplot.contourf` draw contour lines and
         filled contours, respectively.  Except as noted, function
         signatures and return values are the same for both versions.
 
-        :func:`~matplotlib.pyplot.contourf` differs from the MATLAB
-        version in that it does not draw the polygon edges.
-        To draw edges, add line contours with
-        calls to :func:`~matplotlib.pyplot.contour`.
 
+        Parameters
+        ----------
+        X, Y : array-like, optional
+            The coordinates of the values in *Z*.
 
-        Call signatures::
+            *X* and *Y* must both be 2-D with the same shape as *Z* (e.g.
+            created via :func:`numpy.meshgrid`), or they must both be 1-D such
+            that ``len(X) == M`` is the number of columns in *Z* and
+            ``len(Y) == N`` is the number of rows in *Z*.
 
-          contour(Z)
+            If not given, they are assumed to be integer indices, i.e.
+            ``X = range(M)``, ``Y = range(N)``.
 
-        make a contour plot of an array *Z*. The level values are chosen
-        automatically.
+        Z : array-like(N, M)
+            The height values over which the contour is drawn.
 
-        ::
+        levels : int or array-like, optional
+            Determines the number and positions of the contour lines / regions.
 
-          contour(X,Y,Z)
+            If an int *n*, use *n* data intervals; i.e. draw *n+1* contour
+            lines. The level heights are automatically chosen.
 
-        *X*, *Y* specify the (x, y) coordinates of the surface
+            If array-like, draw contour lines at the specified levels.
+            The values must be in increasing order.
 
-        ::
+        Returns
+        -------
+        c : `~.contour.QuadContourSet`
 
-          contour(Z,N)
-          contour(X,Y,Z,N)
-
-        contour up to *N+1* automatically chosen contour levels
-        (*N* intervals).
-
-        ::
-
-          contour(Z,V)
-          contour(X,Y,Z,V)
-
-        draw contour lines at the values specified in sequence *V*,
-        which must be in increasing order.
-
-        ::
-
-          contourf(..., V)
-
-        fill the ``len(V)-1`` regions between the values in *V*,
-        which must be in increasing order.
-
-        ::
-
-          contour(Z, **kwargs)
-
-        Use keyword args to control colors, linewidth, origin, cmap ... see
-        below for more details.
-
-        *X* and *Y* must both be 2-D with the same shape as *Z*, or they
-        must both be 1-D such that ``len(X)`` is the number of columns in
-        *Z* and ``len(Y)`` is the number of rows in *Z*.
-
-        ``C = contour(...)`` returns a
-        :class:`~matplotlib.contour.QuadContourSet` object.
-
-        Optional keyword arguments:
-
-          *corner_mask*: bool, optional
+        Other Parameters
+        ----------------
+        corner_mask : bool, optional
             Enable/disable corner masking, which only has an effect if *Z* is
             a masked array.  If ``False``, any quad touching a masked point is
             masked out.  If ``True``, only the triangular corners of quads
@@ -1657,50 +1712,54 @@ class QuadContourSet(ContourSet):
             Defaults to ``rcParams['contour.corner_mask']``, which defaults to
             ``True``.
 
-          *colors*: [ *None* | string | (mpl_colors) ]
-            If *None*, the colormap specified by cmap will be used.
+        colors : color string or sequence of colors, optional
+            The colors of the levels, i.e. the lines for `.contour` and the
+            areas for `.contourf`.
 
-            If a string, like 'r' or 'red', all levels will be plotted in this
-            color.
+            The sequence is cycled for the levels in ascending order. If the
+            sequence is shorter than the number of levels, it's repeated.
 
-            If a tuple of matplotlib color args (string, float, rgb, etc),
-            different levels will be plotted in different colors in the order
-            specified.
+            As a shortcut, single color strings may be used in place of
+            one-element lists, i.e. ``'red'`` instead of ``['red']`` to color
+            all levels with the same color. This shortcut does only work for
+            color strings, not for other ways of specifying colors.
 
-          *alpha*: float
-            The alpha blending value
+            By default (value *None*), the colormap specified by *cmap*
+            will be used.
 
-          *cmap*: [ *None* | Colormap ]
-            A cm :class:`~matplotlib.colors.Colormap` instance or
-            *None*. If *cmap* is *None* and *colors* is *None*, a
-            default Colormap is used.
+        alpha : float, optional
+            The alpha blending value, between 0 (transparent) and 1 (opaque).
 
-          *norm*: [ *None* | Normalize ]
-            A :class:`matplotlib.colors.Normalize` instance for
-            scaling data values to colors. If *norm* is *None* and
-            *colors* is *None*, the default linear scaling is used.
+        cmap : str or `.Colormap`, optional
+            A `.Colormap` instance or registered colormap name. The colormap
+            maps the level values to colors.
+            Defaults to :rc:`image.cmap`.
 
-          *vmin*, *vmax*: [ *None* | scalar ]
-            If not *None*, either or both of these values will be
-            supplied to the :class:`matplotlib.colors.Normalize`
-            instance, overriding the default color scaling based on
-            *levels*.
+            If given, *colors* take precedence over *cmap*.
 
-          *levels*: [level0, level1, ..., leveln]
-            A list of floating point numbers indicating the level
-            curves to draw, in increasing order; e.g., to draw just
-            the zero contour pass ``levels=[0]``
+        norm : `~matplotlib.colors.Normalize`, optional
+            If a colormap is used, the `.Normalize` instance scales the level
+            values to the canonical colormap range [0, 1] for mapping to
+            colors. If not given, the default linear scaling is used.
 
-          *origin*: [ *None* | 'upper' | 'lower' | 'image' ]
-            If *None*, the first value of *Z* will correspond to the
-            lower left corner, location (0,0). If 'image', the rc
-            value for ``image.origin`` will be used.
+        vmin, vmax : float, optional
+            If not *None*, either or both of these values will be supplied to
+            the `.Normalize` instance, overriding the default color scaling
+            based on *levels*.
 
-            This keyword is not active if *X* and *Y* are specified in
-            the call to contour.
+        origin : {*None*, 'upper', 'lower', 'image'}, optional
+            Determines the orientation and exact position of *Z* by specifying
+            the position of ``Z[0, 0]``.  This is only relevant, if *X*, *Y*
+            are not given.
 
-          *extent*: [ *None* | (x0,x1,y0,y1) ]
+            - *None*: ``Z[0, 0]`` is at X=0, Y=0 in the lower left corner.
+            - 'lower': ``Z[0, 0]`` is at X=0.5, Y=0.5 in the lower left corner.
+            - 'upper': ``Z[0, 0]`` is at X=N+0.5, Y=0.5 in the upper left
+              corner.
+            - 'image': Use the value from :rc:`image.origin`. Note: The value
+              *None* in the rcParam is currently handled as 'lower'.
 
+        extent : (x0, x1, y0, y1), optional
             If *origin* is not *None*, then *extent* is interpreted as
             in :func:`matplotlib.pyplot.imshow`: it gives the outer
             pixel boundaries. In this case, the position of Z[0,0]
@@ -1711,31 +1770,58 @@ class QuadContourSet(ContourSet):
             This keyword is not active if *X* and *Y* are specified in
             the call to contour.
 
-          *locator*: [ *None* | ticker.Locator subclass ]
-            If *locator* is *None*, the default
-            :class:`~matplotlib.ticker.MaxNLocator` is used. The
-            locator is used to determine the contour levels if they
-            are not given explicitly via the *V* argument.
+        locator : ticker.Locator subclass, optional
+            The locator is used to determine the contour levels if they
+            are not given explicitly via *levels*.
+            Defaults to `~.ticker.MaxNLocator`.
 
-          *extend*: [ 'neither' | 'both' | 'min' | 'max' ]
-            Unless this is 'neither', contour levels are automatically
-            added to one or both ends of the range so that all data
-            are included. These added ranges are then mapped to the
-            special colormap values which default to the ends of the
-            colormap range, but can be set via
-            :meth:`matplotlib.colors.Colormap.set_under` and
-            :meth:`matplotlib.colors.Colormap.set_over` methods.
+        extend : {'neither', 'both', 'min', 'max'}, optional, default: \
+'neither'
+            Determines the ``contourf``-coloring of values that are outside the
+            *levels* range.
 
-          *xunits*, *yunits*: [ *None* | registered units ]
+            If 'neither', values outside the *levels* range are not colored.
+            If 'min', 'max' or 'both', color the values below, above or below
+            and above the *levels* range.
+
+            Values below ``min(levels)`` and above ``max(levels)`` are mapped
+            to the under/over values of the `.Colormap`. Note, that most
+            colormaps do not have dedicated colors for these by default, so
+            that the over and under values are the edge values of the colormap.
+            You may want to set these values explicitly using
+            `.Colormap.set_under` and `.Colormap.set_over`.
+
+            .. note::
+
+                An exising `.QuadContourSet` does not get notified if
+                properties of its colormap are changed. Therefore, an explicit
+                call `.QuadContourSet.changed()` is needed after modifying the
+                colormap. The explicit call can be left out, if a colorbar is
+                assigned to the `.QuadContourSet` because it interally calls
+                `.QuadContourSet.changed()`.
+
+            Example::
+
+                x = np.arange(1, 10)
+                y = x.reshape(-1, 1)
+                h = x * y
+
+                cs = plt.contourf(h, levels=[10, 30, 50],
+                    colors=['#808080', '#A0A0A0', '#C0C0C0'], extend='both')
+                cs.cmap.set_over('red')
+                cs.cmap.set_under('blue')
+                cs.changed()
+
+        xunits, yunits : registered units, optional
             Override axis units by specifying an instance of a
             :class:`matplotlib.units.ConversionInterface`.
 
-          *antialiased*: bool
-            enable antialiasing, overriding the defaults.  For
+        antialiased : bool, optinal
+            Enable antialiasing, overriding the defaults.  For
             filled contours, the default is *True*.  For line contours,
-            it is taken from rcParams['lines.antialiased'].
+            it is taken from :rc:`lines.antialiased`.
 
-          *nchunk*: [ 0 | integer ]
+        Nchunk : int >= 0, optional
             If 0, no subdivision of the domain.  Specify a positive integer to
             divide the domain into subdomains of *nchunk* by *nchunk* quads.
             Chunking reduces the maximum length of polygons generated by the
@@ -1744,43 +1830,52 @@ class QuadContourSet(ContourSet):
             however introduce rendering artifacts at chunk boundaries depending
             on the backend, the *antialiased* flag and value of *alpha*.
 
-        contour-only keyword arguments:
+        linewidths : float or sequence of float, optional
+            *Only applies to* `.contour`.
 
-          *linewidths*: [ *None* | number | tuple of numbers ]
-            If *linewidths* is *None*, the default width in
-            ``lines.linewidth`` in ``matplotlibrc`` is used.
+            The line width of the contour lines.
 
             If a number, all levels will be plotted with this linewidth.
 
-            If a tuple, different levels will be plotted with different
-            linewidths in the order specified.
+            If a sequence, the levels in ascending order will be plotted with
+            the linewidths in the order specified.
 
-          *linestyles*: [ *None* | 'solid' | 'dashed' | 'dashdot' | 'dotted' ]
-            If *linestyles* is *None*, the default is 'solid' unless
-            the lines are monochrome.  In that case, negative
-            contours will take their linestyle from the ``matplotlibrc``
-            ``contour.negative_linestyle`` setting.
+            Defaults to :rc:`lines.linewidth`.
+
+        linestyles : {*None*, 'solid', 'dashed', 'dashdot', 'dotted'}, optional
+            *Only applies to* `.contour`.
+
+            If *linestyles* is *None*, the default is 'solid' unless the lines
+            are monochrome.  In that case, negative contours will take their
+            linestyle from :rc:`contour.negative_linestyle` setting.
 
             *linestyles* can also be an iterable of the above strings
             specifying a set of linestyles to be used. If this
             iterable is shorter than the number of contour levels
             it will be repeated as necessary.
 
-        contourf-only keyword arguments:
+        hatches : List[str], optional
+            *Only applies to* `.contourf`.
 
-          *hatches*:
             A list of cross hatch patterns to use on the filled areas.
             If None, no hatching will be added to the contour.
             Hatching is supported in the PostScript, PDF, SVG and Agg
             backends only.
 
 
-        Note: contourf fills intervals that are closed at the top; that
-        is, for boundaries *z1* and *z2*, the filled region is::
+        Notes
+        -----
+        1. :func:`~matplotlib.pyplot.contourf` differs from the MATLAB
+           version in that it does not draw the polygon edges.
+           To draw edges, add line contours with
+           calls to :func:`~matplotlib.pyplot.contour`.
 
-            z1 < z <= z2
+        2. contourf fills intervals that are closed at the top; that
+           is, for boundaries *z1* and *z2*, the filled region is::
 
-        There is one exception: if the lowest boundary coincides with
-        the minimum value of the *z* array, then that minimum value
-        will be included in the lowest interval.
+              z1 < Z <= z2
+
+           There is one exception: if the lowest boundary coincides with
+           the minimum value of the *Z* array, then that minimum value
+           will be included in the lowest interval.
         """
